@@ -40,8 +40,8 @@
  *  sasl_decode       Decode data received using security layer
  *  
  * Utility functions:
- *  sasl_encode64     Encode data to send using security layer
- *  sasl_decode64     Decode data received using security layer
+ *  sasl_encode64     Encode data to send using MIME base64 encoding
+ *  sasl_decode64     Decode data received using MIME base64 encoding
  *  sasl_erasebuffer  Erase a buffer
  *
  * Client Function Summary:
@@ -121,7 +121,7 @@
 
 #define SASL_VERSION_MAJOR 2
 #define SASL_VERSION_MINOR 1
-#define SASL_VERSION_STEP 2
+#define SASL_VERSION_STEP 5
 
 #include "prop.h"
 #define LIBSASL_API
@@ -446,7 +446,7 @@ typedef int sasl_verifyfile_t(void *context,
  * outputs:
  *  result        -- set to NUL terminated string
  *                   NULL = user cancel
- *  len           -- length of result, ignored with SASL_CB_SECRET
+ *  len           -- length of result
  * returns SASL_OK
  */
 typedef int sasl_getsimple_t(void *context, int id,
@@ -613,6 +613,12 @@ typedef int sasl_canon_user_t(sasl_conn_t *conn,
  * Common Client/server functions *
  **********************************/
 
+/* get sasl library version information
+ * implementation is a vendor-defined string
+ * version is a vender-defined representation of the version # */
+LIBSASL_API void sasl_version(const char **implementation,
+			      int *version);
+
 /* dispose of all SASL plugins.  Connection
  * states have to be disposed of before calling this.
  */
@@ -1054,7 +1060,7 @@ LIBSASL_API int sasl_setpass(sasl_conn_t *conn,
 #define SASL_AUX_END      NULL	/* last auxiliary property */
 
 /* traditional Posix items (should be implemented on Posix systems) */
-#define SASL_AUX_PASSWORD "userPassword" /* User Password */
+#define SASL_AUX_PASSWORD "*userPassword" /* User Password (of authid) */
 #define SASL_AUX_UIDNUM   "uidNumber"	/* UID number for the user */
 #define SASL_AUX_GIDNUM   "gidNumber"	/* GID for the user */
 #define SASL_AUX_FULLNAME "gecos"	/* full name of the user, unix-style */
