@@ -736,7 +736,7 @@ typedef struct sasl_canonuser {
     void *glob_context;
 
     /* name of plugin */
-    void *spare_ptr1;
+    char *name;
 
     /* free global state for plugin */
     void (*canon_user_free)(void *glob_context, const sasl_utils_t *utils);
@@ -820,15 +820,21 @@ typedef struct sasl_auxprop_plug {
 			   unsigned flags,
 			   const char *user, unsigned ulen);
 
+    /* name of the auxprop plugin */
+    char *name;
+
     /* for additions which don't require a version upgrade; set to 0 */
     void (*spare_fptr1)();
-    void (*spare_fptr2)();
 } sasl_auxprop_plug_t;
 
 /* auxprop lookup flags */
 #define SASL_AUXPROP_OVERRIDE 0x01 /* if clear, ignore auxiliary properties
 				    * with non-zero len field.  If set,
 				    * override value of those properties */
+#define SASL_AUXPROP_AUTHZID  0x02 /* if clear, we are looking up the
+				    * authid flags (prefixed with *), otherwise
+				    * we are looking up the authzid flags
+				    * (no prefix) */
 
 #define SASL_AUXPROP_PLUG_VERSION 4
 
