@@ -1,7 +1,7 @@
 /* SASL server API implementation
  * Rob Siemborski
  * Tim Martin
- * $Id: server.c,v 1.103 2002/02/21 21:28:16 rjs3 Exp $
+ * $Id: server.c,v 1.111 2002/06/10 15:58:34 rjs3 Exp $
  */
 /* 
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
@@ -335,7 +335,7 @@ static void server_done(void) {
 
   if (mechlist != NULL)
   {
-      m=mechlist->mech_list; /* m point to begging of the list */
+      m=mechlist->mech_list; /* m point to beginning of the list */
 
       while (m!=NULL)
       {
@@ -346,10 +346,6 @@ static void server_done(void) {
 	      prevm->plug->mech_free(prevm->plug->glob_context,
 				     mechlist->utils);
 	  }
-	  
-	  if (prevm->plug->glob_context!=NULL) {
-	      sasl_FREE(prevm->plug->glob_context);
-	  }
 
 	  sasl_FREE(prevm->plugname);	  	  
 	  sasl_FREE(prevm);    
@@ -628,7 +624,7 @@ int sasl_server_init(const sasl_callback_t *callbacks,
     }
 
     /* load internal plugins */
-    sasl_server_add_plugin("EXTERNAL", &external_server_init);
+    sasl_server_add_plugin("EXTERNAL", &external_server_plug_init);
 
     /* delayed loading of plugins? (DSO only, as it doesn't
      * make much [any] sense to delay in the static library case) */
@@ -887,7 +883,7 @@ static int mech_permitted(sasl_conn_t *conn,
 	}
     }
     
-    if (plug == &external_server_mech) {
+    if (!strcasecmp(plug->mech_name, "EXTERNAL")) {
 	/* Special case for the external mechanism */
 	if (conn->props.min_ssf > conn->external.ssf
 	    || ! conn->external.auth_id) {
@@ -1244,6 +1240,12 @@ int sasl_server_step(sasl_conn_t *conn,
 	return SASL_OK;
     }
 
+    /* Don't do another step if the plugin told us that we're done */
+    if (conn->oparams.doneflag) {
+	_sasl_log(conn, SASL_LOG_ERR, "attempting server step after doneflag");
+	return SASL_FAIL;
+    }
+
     if(serverout) *serverout = NULL;
     if(serveroutlen) *serveroutlen = 0;
 
@@ -1276,14 +1278,9 @@ int sasl_server_step(sasl_conn_t *conn,
 
 	if(conn->oparams.user == NULL || conn->oparams.authid == NULL) {
 	    sasl_seterror(conn, 0,
-			  "mech did not call canon_user for both authzid and authid");
+			  "mech did not call canon_user for both authzid " \
+			  "and authid");
 	    ret = SASL_BADPROT;
-	} else if(strcmp(conn->oparams.user, conn->oparams.authid)) {
-	/* If authorization id is not the same as authenticationid, get
-	 * the real auxprops */	
-	    prop_clear(s_conn->sparams->propctx,0);
-	    _sasl_auxprop_lookup(s_conn->sparams, 0,
-				 conn->oparams.user, conn->oparams.ulen);
 	}	
     }
     
@@ -1461,7 +1458,7 @@ static int _sasl_checkpass(sasl_conn_t *conn, const char *service,
     sasl_getopt_t *getopt;
     sasl_server_userdb_checkpass_t *checkpass_cb;
     void *context;
-    const char *mech;
+    const char *mlist, *mech;
     struct sasl_verify_password_s *v;
 
     /* call userdb callback function, if available */
@@ -1477,17 +1474,26 @@ static int _sasl_checkpass(sasl_conn_t *conn, const char *service,
     /* figure out how to check (i.e. auxprop or saslauthd or pwcheck) */
     if (_sasl_getcallback(conn, SASL_CB_GETOPT, &getopt, &context)
             == SASL_OK) {
-        getopt(context, NULL, "pwcheck_method", &mech, NULL);
+        getopt(context, NULL, "pwcheck_method", &mlist, NULL);
     }
 
-    if(!mech) mech = DEFAULT_CHECKPASS_MECH;
+    if(!mlist) mlist = DEFAULT_CHECKPASS_MECH;
 
     result = SASL_NOMECH;
 
-    for (v = _sasl_verify_password; v->name; v++) {
-	if(is_mech(mech, v->name)) {
-	    result = v->verify(conn, user, pass, service, s_conn->user_realm);
-	    break;
+    mech = mlist;
+    while (*mech && result != SASL_OK) {
+	for (v = _sasl_verify_password; v->name; v++) {
+	    if(is_mech(mech, v->name)) {
+		result = v->verify(conn, user, pass, service,
+				   s_conn->user_realm);
+		break;
+	    }
+	}
+	if (result != SASL_OK) {
+	    /* skip to next mech in list */
+	    while (*mech && !isspace((int) *mech)) mech++;
+	    while (*mech && isspace((int) *mech)) mech++;
 	}
     }
 
@@ -1566,7 +1572,7 @@ int sasl_user_exists(sasl_conn_t *conn,
 		     const char *user) 
 {
     int result=SASL_NOMECH;
-    const char *mech;
+    const char *mlist, *mech;
     void *context;
     sasl_getopt_t *getopt;
     struct sasl_verify_password_s *v;
@@ -1582,17 +1588,25 @@ int sasl_user_exists(sasl_conn_t *conn,
     /* figure out how to check (i.e. auxprop or saslauthd or pwcheck) */
     if (_sasl_getcallback(conn, SASL_CB_GETOPT, &getopt, &context)
             == SASL_OK) {
-        getopt(context, NULL, "pwcheck_method", &mech, NULL);
+        getopt(context, NULL, "pwcheck_method", &mlist, NULL);
     }
 
-    if(!mech) mech = DEFAULT_CHECKPASS_MECH;
+    if(!mlist) mlist = DEFAULT_CHECKPASS_MECH;
 
     result = SASL_NOMECH;
 
-    for (v = _sasl_verify_password; v->name; v++) {
-	if(is_mech(mech, v->name)) {
-	    result = v->verify(conn, user, NULL, service, user_realm);
-	    break;
+    mech = mlist;
+    while (*mech && result != SASL_OK) {
+	for (v = _sasl_verify_password; v->name; v++) {
+	    if(is_mech(mech, v->name)) {
+		result = v->verify(conn, user, NULL, service, user_realm);
+		break;
+	    }
+	}
+	if (result != SASL_OK) {
+	    /* skip to next mech in list */
+	    while (*mech && !isspace((int) *mech)) mech++;
+	    while (*mech && isspace((int) *mech)) mech++;
 	}
     }
 
