@@ -1,5 +1,5 @@
 /* dbconverter-2.c -- convert libsasl v1 sasldb's to SASLv2 format
- * $Id: dbconverter-2.c,v 1.4 2002/01/24 01:41:35 rjs3 Exp $
+ * $Id: dbconverter-2.c,v 1.5 2002/03/27 20:11:30 rjs3 Exp $
  * Rob Siemborski
  * based on SASLv1 sasldblistusers
  */
@@ -198,11 +198,7 @@ int listusers(const char *path, listcb_t *cb)
 
 #elif defined(SASL_BERKELEYDB)
 
-#ifdef HAVE_DB3_DB_H
-#include <db3/db.h>
-#else
 #include <db.h>
-#endif
 
 /*
  * Open the database
